import secrets


def get_random_question(redis_client) -> str:
    """Gets a random question from the quiz content."""
    return secrets.choice(list(redis_client.hkeys('Questions')))


def save_last_asked_question_to_redis(
    question: str, user_id: int, redis_client, social_network: str,
) -> None:
    """Saves the user's question number in the database."""
    redis_client.hset('Users', f'user_{social_network}_{user_id}', question)


def get_last_asked_question_from_redis(redis_client, user_id: int, social_network: str) -> str:
    """Gets the last asked question from redis."""
    return redis_client.hget('Users', f'user_{social_network}_{user_id}')


def get_answer_to_last_question_of_user(redis_client, last_asked_question: str) -> str:
    """Gets the text of the answer to the last question asked by the user."""
    return redis_client.hget('Questions', last_asked_question)



from typing import List, Optional

import vk_api
from vk_api.keyboard import VkKeyboardColor
from vk_api.longpoll import VkEventType, VkLongPoll

from quiz_bots import settings
from quiz_bots.bots_helpers import (
    BotKeyboardButton,
    make_keyboard_of_vk,
    send_message_to_vk_chat,
)
from quiz_bots.quiz_helpers import get_clean_text_of_answer
from quiz_bots.redis_utils import (
    get_answer_to_last_question_of_user,
    get_last_asked_question_from_redis,
    get_random_question,
    save_last_asked_question_to_redis,
)

BUTTON_NEW_QUESTION = BotKeyboardButton(text='Новый вопрос', color=VkKeyboardColor.PRIMARY)
BUTTON_GIVE_IN = BotKeyboardButton(text='Сдаться', color=VkKeyboardColor.NEGATIVE)
BUTTON_SCORE = BotKeyboardButton(text='Мой счет', color=VkKeyboardColor.POSITIVE)
BUTTON_QUESTION_IS_MADE_INCORRECTLY = BotKeyboardButton(text='Вопрос составлен неверно', color=VkKeyboardColor.DEFAULT)


def handle_new_question_request(vk, event, redis_client, button_give_in_flag: float = False) -> None:
    """Sends a random question to the user."""
    question = get_random_question(redis_client)
    user_id = event.user_id

    if button_give_in_flag:
        top_buttons = [BUTTON_GIVE_IN, BUTTON_SCORE]
        bottom_buttons: Optional[List[BotKeyboardButton]] = [BUTTON_QUESTION_IS_MADE_INCORRECTLY]
    else:
        top_buttons = [BUTTON_GIVE_IN, BUTTON_SCORE]
        bottom_buttons = None

    send_message_to_vk_chat(
        vk_session=vk,
        user_id=user_id,
        custom_keyboard=make_keyboard_of_vk(
            top_buttons=top_buttons,
            bottom_buttons=bottom_buttons,
        ),
        message_text=question.replace('\n', ' '),
    )

    save_last_asked_question_to_redis(
        question=question,
        user_id=user_id,
        redis_client=redis_client,
        social_network='vk',
    )


def handle_solution_attempt(vk, event, redis_client) -> None:
    """Checks answer of user."""
    user_id = event.user_id
    last_asked_question = get_last_asked_question_from_redis(
        redis_client=redis_client,
        user_id=user_id,
        social_network='vk',
    )
    answer_to_last_question_of_user = get_answer_to_last_question_of_user(
        redis_client=redis_client,
        last_asked_question=last_asked_question,
    )
    if event.text.lower() == get_clean_text_of_answer(answer_to_last_question_of_user):
        top_buttons = [BUTTON_NEW_QUESTION]
        message_text = 'Правильно! Для следующего вопроса нажми «Новый вопрос»'
    else:
        top_buttons = [BUTTON_GIVE_IN, BUTTON_SCORE]
        message_text = 'Неправильно... Попробуешь ещё раз?'

    send_message_to_vk_chat(
        vk_session=vk,
        user_id=user_id,
        custom_keyboard=make_keyboard_of_vk(
            top_buttons=top_buttons,
            bottom_buttons=None,
        ),
        message_text=message_text,
    )


def handle_opportunity_to_give_in(vk, event, redis_client) -> None:
    """Handles clicking the "Surrender" button.

    The bot sends the user an answer to the question
    and sends the next question with the next message.
    """
    user_id = event.user_id
    last_asked_question = get_last_asked_question_from_redis(
        redis_client=redis_client,
        user_id=user_id,
        social_network='vk',
    )
    answer_to_last_question_of_user = get_answer_to_last_question_of_user(
        redis_client=redis_client,
        last_asked_question=last_asked_question,
    )
    send_message_to_vk_chat(
        vk_session=vk,
        user_id=user_id,
        message_text=answer_to_last_question_of_user,
    )
    handle_new_question_request(vk, event, redis_client, button_give_in_flag=True)


def handler_question_is_incorrect(vk, event) -> None:
    """Handles clicking the button 'The question is incorrect'."""
    send_message_to_vk_chat(
        vk_session=vk,
        user_id=event.user_id,
        custom_keyboard=make_keyboard_of_vk(
            top_buttons=[BUTTON_GIVE_IN, BUTTON_SCORE],
            bottom_buttons=None,
        ),
        message_text='Спасибо за фидбек! Ждем ответ на предыдущий вопрос!',
    )


def start_vk_bot(args, redis_client):  # noqa: WPS210, WPS231
    """Launch the VK bot."""
    vk_group_token = settings.vk_group_token
    vk_session = vk_api.VkApi(token=vk_group_token)
    vk = vk_session.get_api()
    longpoll = VkLongPoll(vk_session)
    for event in longpoll.listen():
        if event.type == VkEventType.MESSAGE_NEW and event.to_me:
            if event.text == 'Новый вопрос':
                handle_new_question_request(vk, event, redis_client)
            elif event.text == 'Сдаться':
                handle_opportunity_to_give_in(vk, event, redis_client)
            elif event.text == 'Вопрос составлен неверно':
                handler_question_is_incorrect(vk, event)
            else:
                handle_solution_attempt(vk, event, redis_client)



from dataclasses import dataclass
from typing import List, Optional

import telegram
from vk_api.keyboard import VkKeyboard, VkKeyboardColor
from vk_api.utils import get_random_id


@dataclass
class BotKeyboardButton:
    """Button of bot keyboard."""

    text: str
    color: VkKeyboardColor


def make_keyboard_of_vk(
    top_buttons: List[BotKeyboardButton], bottom_buttons: Optional[List[BotKeyboardButton]],
) -> VkKeyboard:
    """Sends the keyboard to the Vk user."""
    keyboard = VkKeyboard(one_time=True)

    for top_button in top_buttons:
        keyboard.add_button(top_button.text, color=top_button.color)

    if not bottom_buttons:
        return keyboard.get_keyboard()

    keyboard.add_line()

    for bottom_button in bottom_buttons:
        keyboard.add_button(bottom_button.text, color=bottom_button.color)

    return keyboard.get_keyboard()


def send_message_to_vk_chat(
    vk_session,
    user_id: int,
    message_text: str,
    custom_keyboard: VkKeyboard = None,
) -> None:
    """Sends a message to the vk chat."""
    if custom_keyboard:
        vk_session.messages.send(
            peer_id=user_id,
            random_id=get_random_id(),
            keyboard=custom_keyboard,
            message=message_text,
        )
    else:
        vk_session.messages.send(
            peer_id=user_id,
            random_id=get_random_id(),
            message=message_text,
        )


def send_message_to_tg_chat(
    update, message_text: str, custom_keyboard=None,
) -> None:
    """Sends a message to the tg chat."""
    if custom_keyboard:
        update.message.reply_text(
            message_text,
            reply_markup=telegram.ReplyKeyboardMarkup(custom_keyboard),
        )
    else:
        update.message.reply_text(message_text)



import argparse

import redis

from quiz_bots.quiz import send_quiz_content_to_redis
from quiz_bots.settings import db_endpoint, db_password
from quiz_bots.tg_bot import start_tg_bot
from quiz_bots.vk_bot import start_vk_bot


def get_redis_client() -> redis.client.StrictRedis:
    """Connect to Redis."""
    redis_host, redis_port = db_endpoint.split(':')
    redis_password = db_password
    return redis.StrictRedis(
        host=redis_host,
        port=redis_port,
        password=redis_password,
        charset='utf-8',
        decode_responses=True,
    )


def create_parsers() -> argparse.ArgumentParser:
    """Create parsers of cli arguments."""
    parser = argparse.ArgumentParser(description='User database utility')

    subparsers = parser.add_subparsers()
    parser_start_tg_bot = subparsers.add_parser('tg-bot', help='Start tg bot')
    parser_start_tg_bot.set_defaults(func=start_tg_bot)
    parser_start_vk_bot = subparsers.add_parser('vk-bot', help='Start vk bot')
    parser_start_vk_bot.set_defaults(func=start_vk_bot)

    parser_export_quiz_content = subparsers.add_parser(
        'export_quiz_content',
        help='Export quiz content to Redis',
    )
    parser_export_quiz_content.add_argument(
        'quiz_questions_folder_path',
        help='Quiz questions folder path',
    )
    parser_export_quiz_content.set_defaults(func=send_quiz_content_to_redis)
    return parser


def main() -> None:
    """Entry point."""
    args = create_parsers().parse_args()
    redis_client = get_redis_client()
    args.func(args, redis_client)


if __name__ == '__main__':
    main()



import glob
import re
from typing import List, Tuple


def fetch_quiz_content_from_file(filepath: str) -> List[Tuple[str, str]]:
    """Fetch quiz question and answer from file."""
    with open(filepath, 'r', encoding='KOI8-R') as file_with_quiz_content:
        raw_quiz_content = iter(file_with_quiz_content.read().split('\n\n'))

    return [
        (
            re.sub(r'Вопрос \d+:\n', '', text_block),
            re.sub(r'Ответ:\n', '', next(raw_quiz_content)),
        )
        for text_block in raw_quiz_content
        if text_block.startswith('Вопрос')
    ]


def send_quiz_content_to_redis(args, redis_client) -> None:
    """Sends quiz content to Redis."""
    file_paths = list(glob.glob(f'{args.quiz_questions_folder_path}/**/*.txt', recursive=True))
    for file_path in file_paths:
        for question, answer in fetch_quiz_content_from_file(file_path):
            redis_client.hset('Questions', question, answer)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['quiz_bots']

package_data = \
{'': ['*']}

install_requires = \
['environs>=8.0.0,<9.0.0',
 'python-telegram-bot>=12.7,<13.0',
 'redis>=3.5.2,<4.0.0',
 'vk-api>=11.8.0,<12.0.0']

entry_points = \
{'console_scripts': ['quiz-bots = quiz_bots.app:main']}

setup_kwargs = {
    'name': 'quiz-bots',
    'version': '0.2.2',
    'description': 'Quiz bots for TG and VK',
    'long_description': '# Bots for quiz\n\n## Description\n[![Build Status](https://travis-ci.com/velivir/quiz-bots.svg?branch=master)](https://travis-ci.com/velivir/quiz-bots)\n[![Maintainability](https://api.codeclimate.com/v1/badges/7bfc3ff61843cbf93a51/maintainability)](https://codeclimate.com/github/velivir/quiz-bots/maintainability)\n![GitHub](https://img.shields.io/github/license/velivir/quiz-bots)\n![Platform](https://img.shields.io/badge/platform-linux-brightgreen)\n[![wemake-python-styleguide](https://img.shields.io/badge/style-wemake-000000.svg)](https://github.com/wemake-services/wemake-python-styleguide)\n\nThis repository contains Vk and TG bots for the quiz. They get used to the questions and check the correctness of the answers to them.\n\nBot examples:\n* [Tg bot](http://t.me/quiztg_bot)\n* [Vk bot](https://vk.com/club183378823) - write a message "Новый вопрос" to the group\n\n## Table of content\n\n- [Installation](#installation)\n- [How to use](#how-to-use)\n- [License](#license)\n- [Project goal](#project-goal)\n\n## Installation\n* Install using [pip](https://pypi.org/project/quiz-bots/):\n    ```bash\n    pip install quiz-bots\n    ```\n* Create a bot in Telegram via [BotFather](https://t.me/BotFather), and get it API token.\n* Create redis account in [Redislabs](https://redislabs.com/), and after that create [cloud database](https://docs.redislabs.com/latest/rc/quick-setup-redis-cloud/) (you can choose free plan).\nGet your endpoint database url and port.\n* Create VK\'s group, allow it send messages, and get access token for it.\n* Register environment variables in the operating system:\n\n    ```bash\n    export TELEGRAM_TOKEN=telegram_token\n    export DB_ENDPOINT=redis endpoint\n    export DB_PASSWORD=redis_password\n    export VK_GROUP_TOKEN=token_vkontakte\n    ```\n\n* Put the question files in a folder(sample files are in the repository folder [quiz_files_example](https://github.com/velivir/quiz-bots/tree/master/quiz_files_example)) and export quiz content to Redis:\n\n    ```bash\n    quiz-bots export_quiz_content [path_to_questions_folder]\n    ```\n\n## How to use\nRun TG bot:\n```bash\nquiz-bots tg-bot\n```\nRun VK bot:\n```bash\nquiz-bots vk-bot\n```\n\n## License\n\nThis project is licensed under the MIT License - see the [LICENSE.md](https://github.com/vitaliy-antonov/quiz-bots/blob/master/LICENSE) file for details\n\n## Project Goal\n\nThe code is written for educational purposes on online-course for\nweb-developers [dvmn.org](https://dvmn.org/).\n',
    'author': 'Vitaliy Antonov',
    'author_email': 'vitaliyantonoff@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/velivir/quiz-bots',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
